@@ -15,15 +15,12 @@
 //
 //--------------------------------------------------------------------------------
 
-#define TIMERSTREAM				FALSE
-#define DOUBLEPARENT			TRUE
-
 #include "AppleUSBAudioCommon.h"
 #include "AppleUSBAudioDevice.h"
-#include "AppleUSBAudioEngine.h"
 #include "AppleUSBAudioLevelControl.h"
 #include "AppleUSBAudioMuteControl.h"
 #include "USBAudioObject.h"
+#include "AppleUSBAudioEngine.h"
 
 #include <IOKit/IOLib.h>
 #include <IOKit/IOKitKeys.h>
@@ -32,6 +29,7 @@
 #include <IOKit/audio/IOAudioDefines.h>
 
 #include <IOKit/usb/IOUSBInterface.h>
+#include <IOKit/usb/IOUSBRootHubDevice.h>
 #include <IOKit/IOMessage.h>
 
 #include <libkern/c++/OSCollectionIterator.h>
@@ -73,13 +71,6 @@ void AppleUSBAudioDevice::free () {
         interfaceLock = NULL;
     }
 
-#if TIMERSTREAM
-    if (deviceNotifier) {
-        deviceNotifier->remove ();
-        deviceNotifier = 0;
-    }
-#endif
-
     if (interfaceVendor) {
         interfaceVendor->release ();
         interfaceVendor = 0;
@@ -91,17 +82,17 @@ void AppleUSBAudioDevice::free () {
     }
 
     if (deviceReleaseNumber) {
-        deviceReleaseNumber->release();
+        deviceReleaseNumber->release ();
         deviceReleaseNumber = 0;
     }
 
     if (configurationValue) {
-        configurationValue->release();
+        configurationValue->release ();
         configurationValue = 0;
     }
 
     if (interfaceNumber) {
-        interfaceNumber->release();
+        interfaceNumber->release ();
         interfaceNumber = 0;
     }
 
@@ -114,13 +105,11 @@ void AppleUSBAudioDevice::free () {
     debug2IOLog ("-AppleUSBAudioDevice[%p]::free ()\n", this);
 }
 
-#if DOUBLEPARENT
 bool AppleUSBAudioDevice::requestTerminate (IOService * provider, IOOptionBits options) {
 	debug4IOLog ("+AppleUSBAudioDevice[%p]::requestTerminate (%p, %x)\n", this, provider, options);
 	debug4IOLog ("-AppleUSBAudioDevice[%p]::requestTerminate (%p, %x)\n", this, provider, options);
 	return TRUE;		// it is OK to terminate us
 }
-#endif
 
 bool AppleUSBAudioDevice::start (IOService * provider) {
     OSObject *						obj;
@@ -144,7 +133,6 @@ bool AppleUSBAudioDevice::start (IOService * provider) {
 	// If this is an iSub, we need to not go any further because we don't support it in this driver
 	// This will cause the driver to not load on any device that has _only_ a low frequency effect output terminal
 	FailIf (usbAudio->GetNumOutputTerminals (0, 0) == 1 && usbAudio->GetIndexedOutputTerminalType (0, 0, 0) == OUTPUT_LOW_FREQUENCY_EFFECTS_SPEAKER, Exit);
-       // IOLog ("This isn't an iSub\n");
 
 	err = kIOReturnError;
 	string[0] = 0;
@@ -171,6 +159,7 @@ bool AppleUSBAudioDevice::start (IOService * provider) {
 	}
 
 	setManufacturerName (string);
+	setDeviceTransportType (kIOAudioDeviceTransportTypeUSB);
 
 	FailIf (FALSE == controlInterface->open (this), Exit);
 	// get the IOAudioDevice generic start routine
@@ -211,40 +200,6 @@ Exit:
 	return resultCode;
 }
 
-#if 0
-UInt32 AppleUSBAudioDevice::getNumAttachedAudioDevices () {
-    OSDictionary *				audioDeviceMatchingDict;
-    UInt32 						numAudioDevices;
-
-    debugIOLog ("+AppleUSBAudioDevice::getNumAttachedAudioDevices ()\n");
-    numAudioDevices = 0;
-    audioDeviceMatchingDict = serviceMatching ("IOAudioDevice");
-
-    if (audioDeviceMatchingDict) {
-        OSIterator *	deviceIterator;
-
-        deviceIterator = getMatchingServices (audioDeviceMatchingDict);
-        if (deviceIterator) {
-            IOAudioDevice *device;
-            while (device = (IOAudioDevice *)deviceIterator->getNextObject ()) {
-                if (device != this) {
-                    numAudioDevices++;
-                }
-            }
-            deviceIterator->release ();
-        } else {
-            debugIOLog ("++AppleUSBAudioDevice::getNumAttachedAudioDevices () - no device iterator created\n");
-        }
-        audioDeviceMatchingDict->release ();
-    } else {
-        debugIOLog ("++AppleUSBAudioDevice::getNumAttachedAudioDevices () - no matching dict created\n");
-    }
-
-    debugIOLog ("-AppleUSBAudioDevice::getNumAttachedAudioDevices ()\n");
-    return numAudioDevices;
-}
-#endif
-
 void AppleUSBAudioDevice::streamStopped (IOAudioEngine *audioEngine) {
     // This must be called from within the workLoop
     // The AppleUSBAudioEngine does so from _setState()
@@ -258,174 +213,6 @@ void AppleUSBAudioDevice::streamStopped (IOAudioEngine *audioEngine) {
     debugIOLog ("-AppleUSBAudioDevice::streamStopped ()\n");
 }
 
-#if TIMERSTREAM
-bool AppleUSBAudioDevice::isUnattachedDeviceRegistered () {
-    debugIOLog ("AppleUSBAudioDevice::isUnattachedDeviceRegistered ()\n");
-    return (deviceNotifier != NULL);
-}
-
-void AppleUSBAudioDevice::registerUnattachedDevice () {
-    debugIOLog ("+AppleUSBAudioDevice::registerUnattachedDevice ()\n");
-    if (!deviceNotifier) {
-        OSDictionary *	matchingDict;
-
-        matchingDict = serviceMatching ("IOUSBInterface");
-        matchingDict->setObject (kUSBVendorName, interfaceVendor);
-        matchingDict->setObject (kUSBProductName, interfaceProduct);
-        matchingDict->setObject (kUSBDeviceReleaseNumber, deviceReleaseNumber);
-        matchingDict->setObject (kUSBConfigurationValue, configurationValue);
-        matchingDict->setObject (kUSBInterfaceNumber, interfaceNumber);
-        matchingDict->setObject (gIOMatchCategoryKey, (OSSymbol *)gIODefaultMatchCategoryKey);
-        
-        deviceNotifier = addNotification(gIOPublishNotification,
-                                         matchingDict,
-                                         (IOServiceNotificationHandler) &interfacePublished,
-                                         this,
-                                         NULL,
-                                         1000);
-        // matchingDict->release();
-    } else {
-        debugIOLog ("++AppleUSBAudioDevice::registerUnattachedDevice () - error - already have a device notifier - inconsistent state.\n");
-    }
-
-    debugIOLog ("-AppleUSBAudioDevice::registerUnattachedDevice ()\n");
-}
-
-void AppleUSBAudioDevice::unregisterUnattachedDevice () {
-    debugIOLog("+AppleUSBAudioDevice::unregisterUnattachedDevice ()\n");
-
-    if (deviceNotifier) {
-        deviceNotifier->remove ();
-        deviceNotifier = 0;
-    } else {
-        debugIOLog("++AppleUSBAudioDevice::unregisterUnattachedDevice () - error - don't have a device notifier - inconsistent state.\n");
-    }
-
-    debugIOLog("-AppleUSBAudioDevice::unregisterUnattachedDevice ()\n");
-}
-
-bool AppleUSBAudioDevice::interfacePublished (AppleUSBAudioDevice *audioDevice, void *ref, IOService *newService) {
-    IOUSBInterface *			interface;
-    bool						result;
-    IOCommandGate		*cg;
-
-    result = TRUE;
-
-    debugIOLog ("+AppleUSBAudioDevice::interfacePublished ()\n");
-
-    if (interface = OSDynamicCast (IOUSBInterface, newService)) {
-        cg = audioDevice->getCommandGate();
-        if (cg) {
-            cg->runAction(reinitWithInterfaceAction, interface, (void *)&result);
-        }
-    }
-    
-    debugIOLog ("-AppleUSBAudioDevice::interfacePublished ()\n");
-    return result;
-}
-
-IOReturn AppleUSBAudioDevice::reinitWithInterfaceAction(OSObject *owner, void *arg1, void *arg2, void *arg3, void *arg4)
-{
-    IOReturn result = kIOReturnBadArgument;
-    
-    if (owner && arg1 && arg2) {
-        AppleUSBAudioDevice *audioDevice = OSDynamicCast(AppleUSBAudioDevice, owner);
-        IOUSBInterface *interface = OSDynamicCast(IOUSBInterface, (OSObject *)arg1);
-        
-        if (audioDevice) {
-            *(bool *)arg2 = audioDevice->reinitWithInterface(interface);
-            result = kIOReturnSuccess;
-        }
-    }
-    
-    return result;
-}
-
-bool AppleUSBAudioDevice::reinitWithInterface (IOUSBInterface *interface) {
-    OSCollectionIterator *			audioEngineIterator;
-    AppleUSBAudioEngine *			audioEngine;
-    OSCollectionIterator *			portIterator;
-    IOAudioPort *					audioPort;
-    OSCollectionIterator *			controlIterator;
-    IOAudioControl *				audioControl;
-    AppleUSBAudioLevelControl *		levelControl;
-    AppleUSBAudioMuteControl *		muteControl;
-    bool							result;
-
-    debug3IOLog("+AppleUSBAudioDevice[%p]::reinitWithInterface(%p)\n", this, interface);
-
-    result = FALSE;
-    FailIf (interface == previousInterface, Exit);
-    FailIf (kIOReturnSuccess != interface->open (this), Exit);
-
-    debug3IOLog("+AppleUSBAudioDevice[%p]::reinitWithInterface(%p) - 1\n", this, interface);
-    
-    controlInterface = interface;
-    previousInterface = interface;
-    this->attach (controlInterface);
-
-    unregisterUnattachedDevice ();
-
-    if (audioEngines) {
-        // We need to do this stuff on the IOWorkLoop
-        
-        // need to tell streams not to notify us when stopped - there's a potential race here
-        // because we've already committed to the new interface, but haven't told the streams
-        // yet - we could still get a notification which could cause us to stop
-        // probably need another lock to make sure that doesn't happen
-
-        audioEngineIterator = OSCollectionIterator::withCollection (audioEngines);
-        if (audioEngineIterator) {
-            while (audioEngine = (AppleUSBAudioEngine *)audioEngineIterator->getNextObject ()) {
-                if (OSDynamicCast (AppleUSBAudioEngine, audioEngine)) {
-                    audioEngine->notifyDeviceOfStop = FALSE;
-                }
-            }
-
-            audioEngineIterator->release ();
-        }
-    }
-
-    debug3IOLog("+AppleUSBAudioDevice[%p]::reinitWithInterface(%p) - 4\n", this, interface);
-    
-    // Force control updates so that the device is set to the correct volume, etc...
-    if (audioPorts) {
-        // No portsLock anymore - need to be on IOWorkLoop
-
-        portIterator = OSCollectionIterator::withCollection (audioPorts);
-        if (portIterator) {
-            while (audioPort = (IOAudioPort *)portIterator->getNextObject ()) {
-                if (OSDynamicCast (IOAudioPort, audioPort)) {
-                    if (audioPort->audioControls) {
-                        controlIterator = OSCollectionIterator::withCollection (audioPort->audioControls);
-
-                        if (controlIterator) {
-                            while (audioControl = (IOAudioControl *)controlIterator->getNextObject ()) {
-                                if (levelControl = OSDynamicCast (AppleUSBAudioLevelControl, audioControl)) {
-                                    levelControl->updateUSBValue ();
-                                } else if (muteControl = OSDynamicCast (AppleUSBAudioMuteControl, audioControl)) {
-                                    muteControl->updateUSBValue ();
-                                }
-                            }
-                            controlIterator->release ();
-                        }
-                    }
-                }
-            }
-            portIterator->release ();
-        }
-    }
-
-    debug3IOLog ("++AppleUSBAudioDevice[%p]::reinitWithInterface (%p) - successfully attached to new interface\n", this, interface);
-
-    result = TRUE;
-
-Exit:
-    debug3IOLog ("-AppleUSBAudioDevice[%p]::reinitWithInterface (%p)\n", this, interface);
-    return result;
-}
-#endif
-
 bool AppleUSBAudioDevice::allStreamsStopped () {
     OSCollectionIterator *			audioEngineIterator;
     IOAudioEngine *					audioEngine;
@@ -472,94 +259,13 @@ IOReturn AppleUSBAudioDevice::performPowerStateChange (IOAudioDevicePowerState o
 	return result;
 }
 
-#if TIMERSTREAM
-IOReturn AppleUSBAudioDevice::performUnattachedDeviceRegistrationAction (OSObject *owner, void *arg1, void *arg2, void *arg3, void *arg4) {
-	IOReturn						result;
-	AppleUSBAudioDevice *			audioDevice;
-
-	result = kIOReturnBadArgument;
-	if (owner && arg1) {
-		audioDevice = OSDynamicCast (AppleUSBAudioDevice, owner);
-		if (audioDevice) {
-			*(bool *)arg1 = audioDevice->performUnattachedDeviceRegistration ();
-			result = kIOReturnSuccess;
-		}
-	}
-
-	return result;
-}
-
-bool AppleUSBAudioDevice::performUnattachedDeviceRegistration () {
-	OSCollectionIterator 		*audioEngineIterator;
-	AppleUSBAudioEngine			*audioEngine;
-	bool 						isRegistered;
-
-	isRegistered = FALSE;
-
-	// We should stick around if we have a running audio engine
-	if (audioEngines) {
-		debugIOLog("++AppleUSBAudioDevice::performUnattachedDeviceRegistration () - we're the only device - checking to see if any audioEngines are running.\n");
-
-		// We're stopping - do we need to lock this stuff? on IOWorkLoop?
-		audioEngineIterator = OSCollectionIterator::withCollection (audioEngines);
-		if (audioEngineIterator) {
-			while (audioEngine = (AppleUSBAudioEngine *)audioEngineIterator->getNextObject ()) {
-				if (OSDynamicCast (AppleUSBAudioEngine, audioEngine)) {	// We only care about runninng AppleUSBAudioEngine objects
-
-					// Nothing more to lock here - definitely need to be on the IOWorkLoop
-					if (audioEngine->getState () == kIOAudioEngineRunning) {
-						debug2IOLog ("+++AppleUSBAudioDevice::performUnattachedDeviceRegistration () - found a running audio engine: %p - sticking around until a re-attach or all audio engines stop.\n", audioEngine);
-						isRegistered = TRUE;
-
-						audioEngine->notifyDeviceOfStop = TRUE;
-
-						if (!isUnattachedDeviceRegistered ()) {
-							// register for new USB device notification
-							registerUnattachedDevice ();
-							if (!hasBeenUnattached) {
-								retain ();
-								hasBeenUnattached = TRUE;
-							}
-						}
-						break;
-					}
-				}
-			}
-			audioEngineIterator->release ();
-		}
-	}
-
-	return isRegistered;
-}
-#endif
-
 void AppleUSBAudioDevice::stop (IOService *provider) {
 	bool			shouldStop;
 
-#if TIMERSTREAM
-	debug6IOLog ("+AppleUSBAudioDevice[%p]::stop (%p) - audioEngines = %p - deviceNotifier = %p - rc=%d\n", this, provider, audioEngines, deviceNotifier, getRetainCount());
-#else
 	debug5IOLog ("+AppleUSBAudioDevice[%p]::stop (%p) - audioEngines = %p - rc=%d\n", this, provider, audioEngines, getRetainCount());
-#endif
 
 	shouldStop = TRUE;
 
-#if TIMERSTREAM
-	if (!isInactive()) {	// We haven't been terminated, so we need to check to see if we are still running
-		IOCommandGate *cg;
-
-		cg = getCommandGate();
-
-		if (cg) {
-			bool isRegistered;
-
-			if (cg->runAction(performUnattachedDeviceRegistrationAction, (void *)&isRegistered) == kIOReturnSuccess) {
-				shouldStop = !isRegistered;
-			}
-		}
-	}
-#endif
-
 	if (shouldStop) {
 		performStop (provider);
 	}
@@ -574,7 +280,6 @@ Boolean AppleUSBAudioDevice::ShouldUpdatePRAM (void) {
 	IORegistryEntry *				usbRegEntry;
 	OSObject *						obj;
 	OSNumber *						number;
-	UInt32							locationID;
 	UInt16							productID;
 	UInt16							vendorID;
 	Boolean							speakersGood;
@@ -614,12 +319,10 @@ Boolean AppleUSBAudioDevice::ShouldUpdatePRAM (void) {
 			number = OSDynamicCast (OSNumber, obj);
 			FailIf (NULL == number, Exit);
 
-			locationID = number->unsigned32BitValue ();
-			debug2IOLog ("speaker's locationID = 0x%lx\n", locationID);
-
-			if (0x18000000 == locationID || 0x19000000 == locationID) {
+			if (OSDynamicCast (IOUSBRootHubDevice, usbRegEntry)) {
 				// It's connected to the root hub
 				connectionGood = TRUE;
+				debugIOLog ("Directly connected to the root hub\n");
 			} else {
 				obj = usbRegEntry->getProperty (kUSBProductID);
 				number = OSDynamicCast (OSNumber, obj);
@@ -631,11 +334,12 @@ Boolean AppleUSBAudioDevice::ShouldUpdatePRAM (void) {
 				if (kStudioDisplay15CRT == productID || kStudioDisplay17CRT == productID || kCinemaDisplay == productID || kStudioDisplay17FP == productID) {
 					// It's connected to a good monitor
 					connectionGood = TRUE;
+					debugIOLog ("Connected to a capable monitor\n");
 				}
 			}
 		}
 	}
-	debug2IOLog ("connectionGood = %d\n", speakersGood);
+	debug2IOLog ("connectionGood = %d\n", connectionGood);
 
 	// And there CANNOT be a "sound" node in the device tree so that OF will boot beep through them
 	if (TRUE == connectionGood && FALSE == FindSoundNode ()) {
@@ -643,7 +347,7 @@ Boolean AppleUSBAudioDevice::ShouldUpdatePRAM (void) {
 	}
 
 Exit:
-	debug2IOLog ("- ShouldUpdatePRAM result = %d", result);
+	debug2IOLog ("- ShouldUpdatePRAM result = %d\n", result);
 	return result;
 }
 
@@ -670,7 +374,7 @@ Boolean AppleUSBAudioDevice::FindSoundNode (void) {
 		if (0 == strcmp (name, "mac-io")) {
 			// This is where we want to start the search
 			iterator->release ();		// release the current iterator and make a new one rooted at "mac-io"
-			iterator = IORegistryIterator::iterateOver (regEntry, dtPlane, kIORegistryIterateRecursively);
+			iterator = IORegistryIterator::iterateOver (regEntry, dtPlane);
 			done = TRUE;
 		}
 		regEntry = iterator->getNextObject ();
@@ -687,6 +391,7 @@ Boolean AppleUSBAudioDevice::FindSoundNode (void) {
 	}
 
 	iterator->release ();
+
 Exit:
 	return found;
 }
@@ -695,13 +400,6 @@ void AppleUSBAudioDevice::performStop (IOService *provider) {
 
     debug3IOLog("+AppleUSBAudioDevice[%p]::performStop (%p)\n", this, provider);
 
-#if TIMERSTREAM
-    // remove from new USB device notification
-    if (isUnattachedDeviceRegistered ()) {
-        unregisterUnattachedDevice ();
-    }
-#endif
-
 	super::stop (provider);  // call the IOAudioDevice generic stop routine
 
     if (controlInterface) {
@@ -709,21 +407,10 @@ void AppleUSBAudioDevice::performStop (IOService *provider) {
         controlInterface = NULL;
     }
 
-#if TIMERSTREAM
-    if (hasBeenUnattached) {
-        // we need to schedule this for later
-        release ();
-    }
-#endif
-
     debug3IOLog ("-AppleUSBAudioDevice[%p]::performStop (%p)\n", this, provider);
 }
 
-bool AppleUSBAudioDevice::terminate (IOOptionBits options = 0) {
-#if TIMERSTREAM
-	IOCommandGate *					cg;
-	bool							isRegistered;
-#endif
+bool AppleUSBAudioDevice::terminate (IOOptionBits options) {
 	bool							shouldTerminate;
 	bool							result;
 
@@ -731,16 +418,6 @@ bool AppleUSBAudioDevice::terminate (IOOptionBits options = 0) {
 	result = TRUE;
 	debug3IOLog ("+AppleUSBAudioDevice[%p]::terminate () - rc=%d\n", this, getRetainCount ());
 
-#if TIMERSTREAM
-	cg = getCommandGate();
-
-	if (cg) {
-		if (cg->runAction (performUnattachedDeviceRegistrationAction, (void *)&isRegistered) == kIOReturnSuccess) {
-			shouldTerminate = !isRegistered;
-		}
-	}
-#endif
-
 	if (shouldTerminate) {
 		result = super::terminate (options);
 	}
@@ -756,12 +433,6 @@ bool AppleUSBAudioDevice::finalize (IOOptionBits options) {
 
 	result = super::finalize (options);
 
-#if TIMERSTREAM
-	if (isUnattachedDeviceRegistered ()) {
-		stop (0);
-	}
-#endif
-
 	debug4IOLog ("-AppleUSBAudioDevice[%p]::finalize (%p) - rc=%d\n", this, options, getRetainCount ());
 
 	return result;
@@ -790,6 +461,7 @@ IOReturn AppleUSBAudioDevice::activateAudioEngine (IOAudioEngine *audioEngine, b
     UInt8								channelNum;
     AppleUSBAudioLevelControl *			speakerControl;
     AppleUSBAudioLevelControl *			micControl;
+	AppleUSBAudioLevelControl *			pramControl;
     AppleUSBAudioMuteControl *			muteControl;
     IOReturn							result;
 	UInt16								terminalType;
@@ -829,7 +501,24 @@ IOReturn AppleUSBAudioDevice::activateAudioEngine (IOAudioEngine *audioEngine, b
 		FailWithAction (0 == featureUnitID, result = kIOReturnSuccess, Finish);	// There isn't a feature unit connected to this input terminal
 		// The interface was opened in AppleUSBAudioEngine::initHardware
 		controlInterface->SetAlternateInterface (this, kRootAlternateSetting);
+
 		shouldUpdatePRAM = ShouldUpdatePRAM ();
+		if (shouldUpdatePRAM) {
+			pramControl = AppleUSBAudioLevelControl::create (featureUnitID,
+															controlInterfaceNum,
+															0xff,
+															1,
+															shouldUpdatePRAM,
+															(USBDeviceRequest)&deviceRequest,
+															this,
+															kIOAudioLevelControlSubTypeVolume,
+															kIOAudioControlUsageOutput);
+			if (NULL != pramControl) {
+				audioEngine->addDefaultAudioControl (pramControl);
+				pramControl->release ();
+			}
+		}
+
 		for (channelNum = 0; channelNum <= usbAudio->GetNumChannels (interfaceNum, altInterfaceNum); channelNum++) {
 			if (usbAudio->ChannelHasVolumeControl (controlInterfaceNum, 0, featureUnitID, channelNum)) {
 				speakerControl = AppleUSBAudioLevelControl::create (featureUnitID,
@@ -931,6 +620,8 @@ Finish:
 	result = TRUE;
 
 Exit:
+	flushAudioControls ();
+
     debug4IOLog ("-AppleUSBAudioDevice[%p]::activateAudioEngine (%p) - rc=%d\n", this, audioEngine, getRetainCount ());
     return result;
 }
