@@ -11,6 +11,8 @@
 #ifndef _APPLEUSBAUDIOENGINE_H
 #define _APPLEUSBAUDIOENGINE_H
 
+#define NEWUSB					TRUE
+
 #include <IOKit/IOBufferMemoryDescriptor.h>
 #include <IOKit/audio/IOAudioEngine.h>
 #include <IOKit/audio/IOAudioStream.h>
@@ -34,16 +36,14 @@ class AppleUSBAudioDevice;
 
 #define kMinimumFrameOffset				1
 
-// Don't make NUM_FRAME_LISTS * NUM_FRAMES_PER_LIST smaller than 375ms or else the HAL will have problems
-#define NUM_FRAME_LISTS					40
-#if FIXEDSIZEPACKETS
-#define NUM_FRAMES_PER_LIST				10
-#else
-#define NUM_FRAMES_PER_LIST				64
-#endif
-#define NUM_FRAME_LISTS_TO_QUEUE		10
 
-#define MAX_WRITE_RETRIES				10
+#define RECORD_NUM_USB_FRAME_LISTS				128
+#define RECORD_NUM_USB_FRAMES_PER_LIST			2
+#define RECORD_NUM_USB_FRAME_LISTS_TO_QUEUE		64
+
+#define PLAY_NUM_USB_FRAME_LISTS				4
+#define PLAY_NUM_USB_FRAMES_PER_LIST			64
+#define PLAY_NUM_USB_FRAME_LISTS_TO_QUEUE		2
 
 class IOSyncer;
 class AppleUSBAudioEngine;
@@ -73,30 +73,45 @@ public:
     
     virtual IOAudioEngineState _setState (IOAudioEngineState newState);
 
-    static void readHandler (void * object, void * frameListIndex, IOReturn result, IOUSBIsocFrame * pFrames);
 	static void sampleRateHandler (void * target, void * parameter, IOReturn result, IOUSBIsocFrame * pFrames);
+
+#if NEWUSB
+    static void readHandler (void * object, void * frameListIndex, IOReturn result, IOUSBLowLatencyIsocFrame * pFrames);
+    static void writeHandler (void * object, void * parameter, IOReturn result, IOUSBLowLatencyIsocFrame * pFrames);
+#else
+    static void readHandler (void * object, void * frameListIndex, IOReturn result, IOUSBIsocFrame * pFrames);
     static void writeHandler (void * object, void * parameter, IOReturn result, IOUSBIsocFrame * pFrames);
+#endif
+
 	static IOReturn iSubAttachChangeHandler (IOService *target, IOAudioControl *passThruControl, SInt32 oldValue, SInt32 newValue);
 
 protected:
-	UInt64					theFirstFrame;
+	UInt64								usbFrameToQueueAt;
 	UInt64					nextSynchReadFrame;
-	IOUSBIsocFrame *		theFrames;
+#if NEWUSB
+	IOUSBLowLatencyIsocFrame *			theUSBIsocFrames;
+	IOUSBLowLatencyIsocFrame			theStartFrame;
+#else
+	IOUSBIsocFrame *					theUSBIsocFrames;
+	IOUSBIsocFrame						theStartFrame;
+#endif
 	IOUSBIsocFrame			theSampleRateFrame;
-	IOUSBIsocFrame			theStartFrame;
+#if NEWUSB
+	IOUSBLowLatencyIsocCompletion *		usbCompletion;
+#else
 	IOUSBIsocCompletion *	usbCompletion;
-	IOUSBIsocCompletion		theStartCompletion;
+#endif
 	IOUSBIsocCompletion		sampleRateCompletion;
-	IOSyncer *				signal;
-	AppleUSBAudioDevice *	usbAudioDevice;
 	IOUSBInterface *		streamInterface;
 	IOUSBInterface *		previousInterface;
 	IOUSBPipe *				thePipe;
 	IOUSBPipe *				theAssociatedPipe;
-	IOMemoryDescriptor **	soundBuffer;
+	IOSyncer *							signal;
+	AppleUSBAudioDevice *				usbAudioDevice;
+	IOMemoryDescriptor **				soundBufferDescriptors;
 	IOMemoryDescriptor *	theStartBufferDescriptor;
 	IOMemoryDescriptor *	neededSampleRate;
-	FrameListWriteInfo *	pendingFrameListWriteInfo;
+	IOVirtualRange						theWrapRanges[2];
 	IONotifier *			streamNotifier;
 	OSObject *				interfaceVendor;
 	OSObject *				interfaceProduct;
@@ -105,33 +120,34 @@ protected:
 	OSObject *				interfaceNumber;
 	void *					readBuffer;
 	void *					theStartBuffer;
-	thread_call_t			pendingFrameListCall;
+	UInt32 *							aveSampleRateBuf;		// only needs to be 3 bytes for the 10.14 value
+#if !NEWUSB
+	UInt64								expectedBusFrame;		// For output, the frame on which we expect to get our next USB isoc completion
+																// For input, the frame on which we last received a USB isoc completion and
+																// wrapped the sample buffer.
+#endif
 	thread_call_t			iSubTeardownThreadCall;
 	thread_call_t			iSubOpenThreadCall;
 	IOAudioSampleRate		curSampleRate;
-	UInt64					expectedBusFrame;		// For output, the frame on which we expect to get our next USB isoc completion
-													// For input, the frame on which we last received a USB isoc completion and
-													// wrapped the sample buffer.
-	UInt64					startTime;
-	UInt64					previousWrappedReadUSBFrame;
+	UInt32								previouslyPreparedBufferOffset;
+	UInt32								safeToEraseTo;
+	UInt32								lastSafeErasePoint;
 	UInt32					theStartBufferSize;
 	UInt32					previous;
 	UInt32					previousClippedToFrame;
 	UInt32					theMaxPacket;
-	UInt32					frameListSize;
-	UInt32					readFrameListSize;
+	UInt32								readUSBFrameListSize;
 	UInt32					bufferOffset;
-	UInt32					aveSampleRateBuf;		// only needs to be 3 bytes for the 10.14 value, but this is more convenient
 	UInt32					averageSampleRate;
 	UInt32					sum;					// This is the number of samples we have sent in the previous frames in this period
-	UInt32					sampleNum;				// The number of the frame that we are currently sending, it goes between 1 and 1000
-	UInt32					numFrameLists;
-	UInt32					numFramesPerList;
-	UInt32					numFrameListsToQueue;
+	UInt32								numUSBFrameLists;
+	UInt32								numUSBFramesPerList;
+	UInt32								numUSBFrameListsToQueue;
 	UInt32					currentFrameList;
 	UInt32					bufferSize;
 	UInt32					bytesPerSampleFrame;
 	UInt32					safetyOffset;
+	UInt32								fractionalSamplesRemaining;
 	UInt16					samplesThisFrame;
 	UInt8					ourInterfaceNumber;
 	UInt8					shouldStop;
@@ -139,6 +155,7 @@ protected:
 	UInt8					refreshInterval;
 	UInt8					framesUntilRefresh;
 	UInt8					direction;
+	Boolean								inCompletion;
 	Boolean					usbStreamRunning;
 	Boolean					notifyDeviceOfStop;
 	Boolean					startingEngine;
@@ -162,13 +179,10 @@ protected:
 	float *					lowFreqSamples;
 	float *					highFreqSamples;
 	PreviousValues				filterState;
-	// aml 2.21.02 adding extra state for 4th order with phase comp filter
 	PreviousValues				filterState2;
 	PreviousValues				phaseCompState;
-	float					srcPhase;			 // aml 3.4.02
-	float					srcState;			 // aml 3.6.02
-       
-        
+	float					srcPhase;			 	
+	float					srcState;			 	
 	Boolean					needToSync;
 
 	static inline IOFixed IOUFixedDivide(UInt32 a, UInt32 b)
@@ -182,10 +196,10 @@ protected:
 	}
 
 	void	GetDeviceInfo (void);
-	IOReturn	PrepareFrameLists (UInt32 numFrameLists, UInt8 sampleSize, UInt8 numChannels);
+	IOReturn	PrepareWriteFrameList (UInt32 usbFrameListIndex);
+	IOReturn	PrepareAndReadFrameLists (UInt8 sampleSize, UInt8 numChannels, UInt32 usbFrameListIndex);
 	IOReturn	SetSampleRate (USBAudioConfigObject *usbAudio, UInt32 sampleRate);
 	IOReturn	AddAvailableFormatsFromDevice (USBAudioConfigObject *usbAudio);
-	UInt32		CalculateNumSamplesPerFrameList (UInt32 sampleRate, UInt32 numFrames, UInt32 theNumFrameLists = 1);
 	IOReturn	CheckForAssociatedEndpoint (USBAudioConfigObject *usbAudio);
 
     static bool audioDevicePublished (AppleUSBAudioEngine *audioEngine, void *ref, IOService *newService);
@@ -212,7 +226,11 @@ protected:
     virtual void *getSampleBuffer (void);
 	virtual UInt32 getSampleBufferSize (void);
 
-	virtual void CoalesceInputSamples (UInt32 numBytesToCoalesce);
+#if NEWUSB
+	virtual void CoalesceInputSamples (UInt32 numBytesToCoalesce, IOUSBLowLatencyIsocFrame * pFrames);
+#else
+	virtual void CoalesceInputSamples (UInt32 numBytesToCoalesce, IOUSBIsocFrame * pFrames);
+#endif
 
 	virtual void resetClipPosition (IOAudioStream *audioStream, UInt32 clipSampleFrame);
     virtual IOReturn clipOutputSamples (const void *mixBuf, void *sampleBuf, UInt32 firstSampleFrame, UInt32 numSampleFrames, const IOAudioStreamFormat *streamFormat, IOAudioStream *audioStream);
